<?php namespace Webinsane\ShopaholicProductCopyist\Behaviors;


use Backend\Classes\ControllerBehavior;
use Exception;
use Lovata\Shopaholic\Models\Offer;
use Lovata\Shopaholic\Models\Product;
use System\Models\File;

class CopyProductBehavior extends ControllerBehavior
{
    public function onCopyProducts()
    {
        $productIds = post('checked') ?: [];
        Product::whereIn('id', $productIds)->get()->each(function(Product $product) {
            $clone = self::cloneProduct($product);
            $clone->save();
        });
        return $this->controller->listRefresh();
    }

    /**
     * @param Product $product
     * @return Product
     */
    private static function cloneProduct(Product $product) {
        $uniqueHash = substr(md5(uniqid(mt_rand(),true)), 0, 4);
        $slug = $product->slug . '_' . $uniqueHash;
        $productCopy = Product::create([
            'slug' => $slug,
            'name' => strpos($product->name, '_copy') ? $product->name : $product->name . '_copy',
            'preview_text' => $product->preview_text,
            'description' => $product->description,
            'active' => $product->active,
            'code' => $product->code,
            'brand_id' => $product->brand_id,
            'category_id' => $product->category_id,
            'external_id' => $product->external_id
        ]);

        $product->additional_category->each(function($category) use ($productCopy) {
          $productCopy->additional_category()->attach($category);
        });

        $productCopy->preview_image = self::getImagePath($product->preview_image);
        self::copyImages($product, $productCopy);

        $product->offer->each(function ($offer) use ($productCopy) {
           self::createOffer($productCopy, $offer);
        });

        return $productCopy;
    }

    private static function copyImages($imagesOwner, $imagesAssignee) {
        $imagesOwner->images->each(function($image) use ($imagesAssignee) {
            $file = new File();
            try {
                $file = $file->fromFile(self::getImagePath($image));
                $file->file_name = $image->getFilename();
                $file->title = $image->title;
                $file->description = $image->description;
                $imagesAssignee->images()->add($file);
            }
            catch (Exception $exception) {
            }
        });
    }

    private static function getImagePath(?File $image)
    {
        if ($image == null) return '';
        return storage_path() . DIRECTORY_SEPARATOR . 'app' . DIRECTORY_SEPARATOR . $image->getDiskPath();
    }

    private static function createOffer(Product $product, Offer $offer)
    {
        $copiedOffer = Offer::create([
            'product_id' => $product->id,
            'active' => $offer->active,
            'name'  => $offer->name,
            'code' => $offer->code,
            'price' => $offer->price,
            'weight' => $offer->weight,
            'quantity' => $offer->quantity,
            'measure' => $offer->measure,
            'height' => $offer->height,
            'length' => $offer->length,
            'width' => $offer->width,
            'quantity_in_unit' => $offer->quantity_in_unit,
            'measure_of_unit' => $offer->measure_of_unit,
            'preview_text' => $offer->preview_text,
            'description' => $offer->description,
        ]);
        $copiedOffer->preview_image = self::getImagePath($offer->preview_image);
        self::copyImages($offer, $copiedOffer);
        $copiedOffer->save();
    }
}